%Made by David Josefsen, spring 2016, for his master's thesis at the
%Norwegian University of Science and Technology, Department of Marine
%Technology


%% Initialization

%Clear out all data and figures for a clean run
clc
clear all
close all

%Initializes the function xlsread1.m which keeps the excel sheet open
%throughout the simulation, instead of opening and closing excel each
%time a parameter is read. This caused tremendous saving in the program 
%run time; After implementing xlsread1.m, the simulation run time was
%reduced to about 22 % of the original simulation time.

%Requires xlsread1.m in the same folder as this program

Excel = actxserver ('Excel.Application'); 
%NB!
%REMEMBER TO DEFINE THE PATH OF THE INPUT DATA FILE BELOW
File='C:\Users\David\Documents\NTNU\MASTER\MATLAB\INPUTDATA.xlsx'; 
if ~exist(File,'file') 
    ExcelWorkbook = Excel.Workbooks.Add; 
    ExcelWorkbook.SaveAs(File,1); 
    ExcelWorkbook.Close(false); 
end 
Excel.Workbooks.Open(File); 

%% Input Data that is valid for each simulation
InitializeRow = 3;
FirstRow = num2str(InitializeRow);
%Fleet Characteristics
%Vessel data that will serve as input
Counter = 1;
FleetRow = FirstRow;
while isempty(xlsread1('INPUTDATA.xlsx',1,['D',FleetRow])) == 0
Fleet(Counter) = xlsread1('INPUTDATA.xlsx',1,['D',FleetRow]);
Counter = Counter + 1;
FleetRow = str2num(FleetRow);
FleetRow = FleetRow + 1;
FleetRow = num2str(FleetRow);
end
EndRow = num2str(str2num(FleetRow) - 1);
%Number of Vessels
nVessels = length(Fleet);
%Max speed [knots]
SpeedMax = xlsread1('INPUTDATA.xlsx',1,['E',FirstRow,':','E',EndRow]);
%Service speed [knots]
SpeedService = xlsread1('INPUTDATA.xlsx',1,['F',FirstRow,':','F',EndRow]);
%Economical speed [knots]
SpeedEco = xlsread1('INPUTDATA.xlsx',1,['G',FirstRow,':','G',EndRow]);
%Fuel consumption in max speed [tonne/hour]
FuelConMax = xlsread1('INPUTDATA.xlsx',1,['H',FirstRow,':','H',EndRow])/24;
%Fuel consumption in service speed [tonne/hour]
FuelConService = xlsread1('INPUTDATA.xlsx',1,['I',FirstRow,':','I',EndRow])/24;
%Fuel consumption in economical speed [tonne/hour]
FuelConEco = xlsread1('INPUTDATA.xlsx',1,['J',FirstRow,':','J',EndRow])/24;
%Fuel consumption in operation [tonne/hour]
FuelConOperation = xlsread1('INPUTDATA.xlsx',1,['K',FirstRow,':','K',EndRow])/24;
%Fuel capacity of the fleet [tonne]
FuelCapacity = xlsread1('INPUTDATA.xlsx',1,['L',FirstRow,':','L',EndRow]);
%Emergency Equipment Status (0 if unequipped, 1 if equipment is available)
EmergencyEquipment = xlsread1('INPUTDATA.xlsx',1,['P',FirstRow,':','P',EndRow]);

%Location of port in latitudes and longitudes
LocationPort = [xlsread1('INPUTDATA.xlsx',1,['S',FirstRow]), xlsread1('INPUTDATA.xlsx',1,['T',FirstRow])];
%Location of oil spill in latidues and longitudes
LocationOilSpill = [xlsread1('INPUTDATA.xlsx',1,['U',FirstRow]), xlsread1('INPUTDATA.xlsx',1,['V',FirstRow])];

%Refueling rate of the port [tonne/hour]
RefuelingRatePort = 200;
%Delay caused by navigation in and out of port [hours]
DelayNavigationPort = 2;

%Read csv datafile into a matrix
Weather = csvread('WeatherData.csv',0,0);

%Choose the number of simulations to repeat
NumberOfSimulations = 10000;
for simulations = 1:NumberOfSimulations
%% Input Data That Changes For Each Simulation

for i=1:nVessels
    %Current fuel level in [tonne] 
    FuelLev(i) = (FuelCapacity(i)-50).*rand(1,1) + 50;
   
    %Operational status
    %0 = unable to abort current operation, 1 = in port, 2 = in 
    %transit going towards platform (or at platform), 3 = return trip
    RndOp = randi(100,1);
    if RndOp <= 2
        OpStatus(i) = 0;
    elseif RndOp > 2 && RndOp <= 51
        OpStatus(i) = 2;
    elseif RndOp > 51 && RndOp <= 100
        OpStatus(i) = 3;
    end
    
    %Cargo status 
    %Does delivery of cargo take priority or conflict with response?
    %0 = unable to do oil spill, 1 able to
    %The probability of Cargo Status = 0 is one in a thousand
    RndCargo = randi(1000,1);
    if RndCargo == 1
        CargoStatus(i) = 0;
    else
        CargoStatus(i) = 1;
    end
    
    %Location of vessels is varied between the coordinates of the port and
    %the oil spill. The sailing route is split into three legs so that the
    %assigned vessel positions are more realistic.
    %The latitude is decided between the port and the oil spill
    Latitude(i) = ([LocationOilSpill(1)] - [LocationPort(1)]).*rand(1,1) + [LocationPort(1)];
    %If the latitude of the vessel position is within the first third of
    %the route (closest to the port), then the longitude can be given as
    %the third of the route that is closest to the port longitudes.
    if Latitude(i) <= ((2/3)*[LocationPort(1)] + (1/3)*[LocationOilSpill(1)]);
        Longitude(i) = (((2/3)*[LocationPort(2)] + (1/3)*[LocationOilSpill(2)]) - [LocationPort(2)]).*rand(1,1) + [LocationPort(2)];
    %If the latitude is within the last third of the route (closest to oil
    %spill), then the longitude is given as the third that is nearest the
    %oil spill longitudes
    elseif Latitude(i) >= ((1/3)*[LocationPort(1)] + (2/3)*[LocationOilSpill(1)])
        Longitude(i) = ([LocationOilSpill(2)]-((1/3)*[LocationPort(2)] + (2/3)*[LocationOilSpill(2)])).*rand(1,1) + (((1/3)*[LocationPort(2)] + (2/3)*[LocationOilSpill(2)]));
    %If the latitude is withing the middle third of the route, then the
    %longitude is chosen accordingly.
    else
        Longitude(i) = (((1/3)*[LocationPort(2)] + (2/3)*[LocationOilSpill(2)]) - ((2/3)*[LocationPort(1)] + (1/3)*[LocationOilSpill(1)])).*rand(1,1) + ((2/3)*[LocationPort(1)] + (1/3)*[LocationOilSpill(1)]);
    end
    %Operational Status is set to 1 (in port) if the vessel latitude is
    %within a 0.05 decimal degrees proximity to the location of the port
    if Latitude(i) <= ([LocationPort(1)] + 0.05)
        OpStatus(i) = 1;
    end
end

% Metocean data
%Read month and day, used for reading the appropriate weather data
%The month is referred to numerically, by a number from 1 to 12, 
%where 1 is January and 12 is December
Month = randi(12,1);
%The day ranges from the 1st to the 31st for the following months
if Month == 1 || Month == 3 || Month == 5 || Month == 7 || Month == 8 || Month == 10 || Month == 12
    Day = randi(31,1);
%The day ranges from the 1st to the 30th for the following months
elseif Month == 4 || Month == 6 || Month == 9 || Month == 11
    Day = randi(30,1);
%The day ranges from the 1st to the 28th for February 
%(leap year is not included)
else
    Day = randi(28,1);
end

%Initialize counter for the number of Hs read
HsNumber = 0;

%Read all significant wave heights for the appropriate month and day in the
%time period 1992-2012 (3 hours interval)
for i = 1:size(Weather,1)
    if (Weather(i,2) == Month) && (Weather(i,3) == Day)
        %Note the number of Hs read
        HsNumber = HsNumber + 1;
        %Read the Hs from the weather data file
        Hsread(HsNumber) = abs(Weather(i,7));
    end
end

%Pick a random Hs from any of the same days from the 21 years of historical
%data
RandomDay = randi(HsNumber,1);

%If Hs is less than or equal to zero, then signal is either not read or
%misread, and another random day will be picked
while Hsread(RandomDay) <= 0
    RandomDay = randi(HsNumber,1);
end

%Set the appropriate significant wave height
Hs = Hsread(RandomDay);

%% Get distance from mapping, calculate sailing time and fuel consumption

%Calculating the distances between vessels and oil spill, and vessels and port (in nautical miles)
for i=1:nVessels
    LocationVessel = [Latitude(i), Longitude(i)];
    %Distance of the great circle route
    %Distance from vessel location to oil spill
    DistOilSpillGC(i) = distance('gc',LocationVessel,LocationOilSpill);
    %Distance from vessel location to port
    DistPortGC(i) = distance('gc',LocationVessel,LocationPort);
    %Distance from port to oil spill
    DistPortOilGC(i) = distance('gc',LocationPort,LocationOilSpill);
    %Rhumb line: DistRL(i) = distance('rh',LocationVessel,LocationOilSpill);
    %Distance in nautical miles
    NMDistOilSpillGC(i) = deg2nm(DistOilSpillGC(i));
    NMDistPortGC(i) = deg2nm(DistPortGC(i));
    NMDistPortOilGC(i) = deg2nm(DistPortOilGC(i));
    
    %Calculate sailing time in hours (affected by Hs)
    %Sailing time to oil spill (max speed)
    SailingTimeOilSpill(i) = ((NMDistOilSpillGC(i)/SpeedMax(i))*1.02^Hs);
    %Sailing time to port (max speed)
    SailingTimePort(i) = ((NMDistPortGC(i)/SpeedMax(i))*1.02^Hs);
    %Sailing time from port to oil spill (max speed)
    SailingTimePortOil(i) = ((NMDistPortOilGC(i)/SpeedMax(i))*1.02^Hs);
    %Sailing time return trip from oil spill to port, in economical mode
    %after oil spill containment operation is complete (economical speed)
    SailingTimeReturn(i) = ((NMDistPortOilGC(i)/(SpeedEco(i)*0.5))*1.02^Hs);
    
    %Calculate fuel consumption for said trip
    %Weather affects sailing time, which again increases fuel consumption
    %Fuel consumption - Vessel location to oil spill
    FuelConsumptionSailToOilSpill(i) = SailingTimeOilSpill(i)*FuelConMax(i);
    %Fuel consumption - Vessel location to port
    FuelConsumptionSailToPort(i) = SailingTimePort(i)*FuelConMax(i);
    %Fuel consumption - Port to oil spill
    FuelConsumptionPortToOilSpill(i) = SailingTimePortOil(i)*FuelConMax(i);
    %Fuel consumption - return trip in fuel save mode
    FuelConsumptionReturn(i) = SailingTimeReturn(i)*FuelConEco(i);
    
    %Time in oil spill operation [hours], very rough estimate
    TimeOperation = 10;
    
    %Time to refuel in port
    TimeRefuelPort(i) = ((FuelCapacity(i) - FuelLev(i) + FuelConsumptionSailToPort(i))/RefuelingRatePort) + DelayNavigationPort;   
end

%% Fuel Criteria

%REFUEL AT OIL SPILL SITE
%Refuel at oil spill site, send other ship to pick up fuel 
%Safety factor of 20 % included
RefuelPort = zeros(nVessels,nVessels);
for i=1:nVessels
    for j=1:nVessels
        if i ~= j
            %Fuel consumed while waiting for other vessel to arrive for refueling
            %That is, fuel consumed while sailing out to oil spill, and
            %fuel consumed in operation at oil spill site while waiting for
            %the refuel ship
            RefuelPort(i,j) = FuelConsumptionSailToOilSpill(i) + ((SailingTimePort(j) + TimeRefuelPort(j) + SailingTimePortOil(j) - SailingTimeOilSpill(i))*FuelConOperation(i));
            if RefuelPort(i,j)*1.2 <= FuelLev(i)
                Refuel(i,j) = j;
            else
                Refuel(i,j) = 0;
            end
        end
    end
end
            
     
%NO REFUELING
for i=1:nVessels
    FuelConsumptionResponse(i) = FuelConsumptionSailToOilSpill(i) + FuelConsumptionReturn(i) + (TimeOperation*FuelConOperation(i));
    if FuelConsumptionResponse(i)*1.2 <= FuelLev(i)
        FuelCriteria(i) = 1;
    else
        FuelCriteria(i) = 0;
    end
end


%% Decide how to respond to the emergency and which vessels to send

%This loop goes through a set of logical decisions and determines 
%the fastest feasible response for each vessel 
for i=1:nVessels
    %Feasibility of vessel (0 if not feasible)
    Vessels(i)=OpStatus(i)*CargoStatus(i)*Fleet(i);
    %Check if vessel is unable to respond
    if Vessels(i) == 0
      Delay = SailingTimePortOil(i)*10;
      ResponseDirect(i) = 0;
      ResponseRefuel(i) = 0;
      ResponseByPort(i) = 0;
    %Check if vessel is feasible for direct response
    elseif Vessels(i) > 0 && FuelCriteria(i) > 0 && EmergencyEquipment(i) ~= 0
        %If vessel is in port, add 4 [hours] delay
        if OpStatus(i) == 1
            Delay = 4;
        %If vessel is in transit towards platform
        elseif OpStatus(i) == 2
            Delay = 0;
        %if vessel is sailing return trip, add 0.2 [hours]
        elseif OpStatus(i) == 3
            Delay = 0.2;
        end
      ResponseDirect(i) = SailingTimeOilSpill(i) + Delay;
      ResponseRefuel(i) = 0;
      ResponseByPort(i) = 0;
    %If the ship's current fuel level is a limiting factor, then
    %check the option of being refueled by another ship while doing oil
    %spill operations
    elseif Vessels(i) > 0 && max(Refuel(i,:)) > 0 && EmergencyEquipment(i) ~= 0
        %If vessel is in port, add 4 [hours] delay
        if OpStatus(i) == 1
            Delay = 4;
        %If vessel is in transit towards platform
        elseif OpStatus(i) == 2
            Delay = 0;
        %if vessel is sailing return trip, add 0.2 [hours] Should also
        %contribute to fuel consumption?
        elseif OpStatus(i) == 3
            Delay = 0.2;
        end
        ResponseDirect(i) = 0;
        ResponseRefuel(i) = SailingTimeOilSpill(i) + Delay;
        ResponseByPort(i) = 0;
    %If the current fuel level is so low that the ship are both unable to
    %perform oil spill operations independently, and unable to wait for
    %another ship to refuel, then it must travel back to port and refuel.
    %This option is also chosen if the vessel does not have the
    %necessary equipment onboard to deal with the emergency, and must thus
    %pick this up at port.
    else
        %If vessel is in port, add 4 [hours] delay
        if OpStatus(i) == 1
            Delay = TimeRefuelPort(i);
            ResponseByPort(i) = SailingTimePortOil(i);
        %If vessel is in transit towards platform, turn around
        elseif OpStatus(i) == 2
            Delay = 0.2;
            ResponseByPort(i) = SailingTimePort(i) + TimeRefuelPort(i) + SailingTimePortOil(i);
        %if vessel is sailing return trip
        elseif OpStatus(i) == 3
            Delay = 0;
            ResponseByPort(i) = SailingTimePort(i) + TimeRefuelPort(i) + SailingTimePortOil(i);
        end
        SailingTimeOilSpill(i) = ResponseByPort(i);
        
        ResponseDirect(i) = 0;
        ResponseRefuel(i) = 0;
        ResponseByPort(i) = SailingTimeOilSpill(i) + Delay;
    end
    ResponseTime(i) = SailingTimeOilSpill(i) + Delay;
end
      
%Choose the minimum response time of all vessels as the solution, 
%and print to command window the time and vessel chosen
      
[MinResponseTime, ChosenVessel] = min(ResponseTime(:));

%Register the result of this particular simulation to a variable containing
%the results of all simulations
AllChosenResponseTimes(simulations) = MinResponseTime;
      
end

disp('Simulations Complete')
disp('All simulated response times can be found in the variable "AllChosenResponseTimes".')

%Closes excel, which have been held open through the entire simulation
Excel.ActiveWorkbook.Save; 
Excel.Quit 
Excel.delete 
clear Excel