%Made by David Josefsen, spring 2016, for his master's thesis at the
%Norwegian University of Science and Technology, Department of Marine
%Technology


%% Initialization

%Clear out all data and figures for a clean run
clc
clear all
close all

%Read csv datafile of metocean data into a matrix
Weather = csvread('WeatherData.csv',0,0);

%Initializes the function xlsread1.m which keeps the excel sheet open
%throughout the simulation, instead of opening and closing excel each
%time a parameter is read. This caused tremendous saving in the program 
%run time.

%Requires xlsread1.m in the same folder as this program

Excel = actxserver ('Excel.Application'); 
%NB!
%REMEMBER TO DEFINE THE PATH OF THE INPUT DATA FILE BELOW
File='C:\Users\David\Documents\NTNU\MASTER\MATLAB\INPUTDATA.xlsx'; 
if ~exist(File,'file') 
    ExcelWorkbook = Excel.Workbooks.Add; 
    ExcelWorkbook.SaveAs(File,1); 
    ExcelWorkbook.Close(false); 
end 
Excel.Workbooks.Open(File);

%% Input Data
%Choose the first row in the data set
InitializeRow = 3;
FirstRow = num2str(InitializeRow);

%Fleet Characteristics

%Vessel data that will serve as input
Counter = 1;
FleetRow = FirstRow;
while isempty(xlsread1('INPUTDATA.xlsx',1,['D',FleetRow])) == 0
Fleet(Counter) = xlsread1('INPUTDATA.xlsx',1,['D',FleetRow]);
Counter = Counter + 1;
FleetRow = str2num(FleetRow);
FleetRow = FleetRow + 1;
FleetRow = num2str(FleetRow);
end
EndRow = num2str(str2num(FleetRow) - 1);
%Number of Vessels
nVessels = length(Fleet);
%Max speed [knots]
SpeedMax = xlsread1('INPUTDATA.xlsx',1,['E',FirstRow,':','E',EndRow]);
%Service speed [knots]
SpeedService = xlsread1('INPUTDATA.xlsx',1,['F',FirstRow,':','F',EndRow]);
%Economical speed [knots]
SpeedEco = xlsread1('INPUTDATA.xlsx',1,['G',FirstRow,':','G',EndRow]);
%Fuel consumption in max speed (14.6 knop) [tonne/hour]
FuelConMax = xlsread1('INPUTDATA.xlsx',1,['H',FirstRow,':','H',EndRow])/24;
%Fuel consumption in service speed (12 knop) [tonne/hour]
FuelConService = xlsread1('INPUTDATA.xlsx',1,['I',FirstRow,':','I',EndRow])/24;
%Fuel consumption in economical speed (10 knots) [tonne/hour]
FuelConEco = xlsread1('INPUTDATA.xlsx',1,['J',FirstRow,':','J',EndRow])/24;
%Fuel consumption in operation [tonne/hour]
FuelConOperation = xlsread1('INPUTDATA.xlsx',1,['K',FirstRow,':','K',EndRow])/24;
%Fuel capacity of the fleet [tonne]
FuelCapacity = xlsread1('INPUTDATA.xlsx',1,['L',FirstRow,':','L',EndRow]);

%Fleet Status

%Current fuel level in [tonne]
FuelLev = xlsread1('INPUTDATA.xlsx',1,['M',FirstRow,':','M',EndRow]);

%Operational status
%0 = unable to abort current operation, 1 = in port, 2 = in 
%transit going towards platform (or at platform), 3 = return trip
OpStatus = xlsread1('INPUTDATA.xlsx',1,['N',FirstRow,':','N',EndRow]);

%Cargo status 
%Does delivery of cargo take priority or conflict with response?
%0 = unable to do oil spill, 1 able to.
CargoStatus = xlsread1('INPUTDATA.xlsx',1,['O',FirstRow,':','O',EndRow]);

%Emergency Equipment Status (0 if unequipped, 1 if equipment is available)
EmergencyEquipment = xlsread1('INPUTDATA.xlsx',1,['P',FirstRow,':','P',EndRow]);

%Locations in latitudes and longitudes
%Location of vessels
Latitude = xlsread1('INPUTDATA.xlsx',1,['Q',FirstRow,':','Q',EndRow]);
Longitude = xlsread1('INPUTDATA.xlsx',1,['R',FirstRow,':','R',EndRow]);
%Location of port
LocationPort = [xlsread1('INPUTDATA.xlsx',1,['S',FirstRow]), xlsread1('INPUTDATA.xlsx',1,['T',FirstRow])];
%Location of oil spill
LocationOilSpill = [xlsread1('INPUTDATA.xlsx',1,['U',FirstRow]), xlsread1('INPUTDATA.xlsx',1,['V',FirstRow])];

%Refueling rate of the port [tonne/hour]
RefuelingRatePort = xlsread1('INPUTDATA.xlsx',1,['W',FirstRow]);
%Delay caused by navigation in and out of port [hours]
DelayNavigationPort = 2;


% Metocean data

%Read month and day, used for reading the appropriate weather data
Month = xlsread1('INPUTDATA.xlsx',1,['B',FirstRow,]);
Day = xlsread1('INPUTDATA.xlsx',1,['C',FirstRow]);

%Initialize counter for the number of Hs read
HsNumber = 0;

%Read all significant wave heights for the appropriate month and day in the
%time period 1992-2012 (3 hours interval)
for i = 1:size(Weather,1)
    %Can also define for the given time of the day
    if (Weather(i,2) == Month) && (Weather(i,3) == Day)
        %Note the number of Hs read
        HsNumber = HsNumber + 1;
        %Read the Hs from the weather data file
        Hsread(HsNumber) = abs(Weather(i,7));
    end
end

%Pick a random Hs from any of the same days from the 21 years of historical
%data
RandomDay = randi(HsNumber,1);

%If Hs is less than or equal to zero, then signal is either not read or
%misread, and another random day will be picked
while Hsread(RandomDay) <= 0
    RandomDay = randi(HsNumber,1);
end

%Set the appropriate significant wave height
Hs = Hsread(RandomDay);

%% Get distance from mapping, calculate sailing time and fuel consumption

%Calculating the distances between vessels and oil spill, and vessels and port (in nautical miles)
for i=1:nVessels
    LocationVessel = [Latitude(i), Longitude(i)];
    %Distance of the great circle route
    %Distance from vessel location to oil spill
    DistOilSpillGC(i) = distance('gc',LocationVessel,LocationOilSpill);
    %Distance from vessel location to port
    DistPortGC(i) = distance('gc',LocationVessel,LocationPort);
    %Distance from port to oil spill
    DistPortOilGC(i) = distance('gc',LocationPort,LocationOilSpill);
    %Rhumb line: DistRL(i) = distance('rh',LocationVessel,LocationOilSpill);
    %Distance in nautical miles
    NMDistOilSpillGC(i) = deg2nm(DistOilSpillGC(i));
    NMDistPortGC(i) = deg2nm(DistPortGC(i));
    NMDistPortOilGC(i) = deg2nm(DistPortOilGC(i));
    

    %Calculate sailing time in hours (affected by Hs)
    %Sailing time to oil spill (max speed)
    SailingTimeOilSpill(i) = ((NMDistOilSpillGC(i)/SpeedMax(i))*1.02^Hs);
    %Sailing time to port (max speed)
    SailingTimePort(i) = ((NMDistPortGC(i)/SpeedMax(i))*1.02^Hs);
    %Sailing time from port to oil spill (max speed)
    SailingTimePortOil(i) = ((NMDistPortOilGC(i)/SpeedMax(i))*1.02^Hs);
    %Sailing time return trip from oil spill to port, in economical speed
    %after oil spill containment operation is complete
    SailingTimeReturn(i) = ((NMDistPortOilGC(i)/(SpeedEco(i)*0.5))*1.02^Hs);
    
    
        %Calculate fuel consumption for said trip
        %Weather affects sailing time, which again increases fuel consumption
    %Fuel consumption - Vessel location to oil spill
    FuelConsumptionSailToOilSpill(i) = SailingTimeOilSpill(i)*FuelConMax(i);
    %Fuel consumption - Vessel location to port
    FuelConsumptionSailToPort(i) = SailingTimePort(i)*FuelConMax(i);
    %Fuel consumption - Port to oil spill
    FuelConsumptionPortToOilSpill(i) = SailingTimePortOil(i)*FuelConMax(i);
    %Fuel consumption - return trip in fuel save mode
    FuelConsumptionReturn(i) = SailingTimeReturn(i)*FuelConEco(i);
    
    %Time in oil spill operation [hours], very rough estimate
    TimeOperation = 10;
    
    %Time to refuel in port
    TimeRefuelPort(i) = ((FuelCapacity(i) - FuelLev(i) + FuelConsumptionSailToPort(i))/RefuelingRatePort) + DelayNavigationPort;
end

%% Fuel Feasibility Criteria

%REFUEL AT OIL SPILL SITE
%Refuel at oil spill site, send other ship to pick up fuel 
%Safety factor of 20 % included
RefuelPort = zeros(nVessels,nVessels);
for i=1:nVessels
    for j=1:nVessels
        if i ~= j
            %Fuel consumed while waiting for other vessel to arrive for refueling
            %That is, fuel consumed while sailing out to oil spill, and
            %fuel consumed in operation at oil spill site while waiting for
            %the fuel support vessel to arrive
            RefuelPort(i,j) = FuelConsumptionSailToOilSpill(i) + ((SailingTimePort(j) + TimeRefuelPort(j) + SailingTimePortOil(j) - SailingTimeOilSpill(i))*FuelConOperation(i));
            if RefuelPort(i,j)*1.2 <= FuelLev(i)
                Refuel(i,j) = j;
            else
                Refuel(i,j) = 0;
            end
        end
    end
end
            
     
%NO REFUELING
for i=1:nVessels
    FuelConsumptionResponse(i) = FuelConsumptionSailToOilSpill(i) + FuelConsumptionReturn(i) + (TimeOperation*FuelConOperation(i));
    if FuelConsumptionResponse(i)*1.2 <= FuelLev(i)
        FuelCriteria(i) = 1;
    else
        FuelCriteria(i) = 0;
    end
end

%% Decide how to respond to the emergency and which vessels to send

%This loop goes through a set of logical decisions and determines 
%the fastest feasible response for each vessel 
for i=1:nVessels
    %Feasibility of vessel (0 if not feasible)
    Vessels(i)=OpStatus(i)*CargoStatus(i)*Fleet(i);
    %Check if vessel is feasible for direct response
    if Vessels(i) > 0 && FuelCriteria(i) > 0 && EmergencyEquipment(i) ~= 0
        %If vessel is in port, add 4 hours delay
        if OpStatus(i) == 1
            Delay = 4;
        %If vessel is in transit towards platform
        elseif OpStatus(i) == 2
            Delay = 0;
        %if vessel is sailing return trip, add 0.2 h. Should also
        %contribute to fuel consumption?
        elseif OpStatus(i) == 3
            Delay = 0.2;
        end
      ResponseDirect(i) = SailingTimeOilSpill(i) + Delay
      ResponseRefuel(i) = 0;
      ResponseByPort(i) = 0;
    %If the ship's current fuel level is a limiting factor, then
    %check the option of being refueled by another ship while doing oil
    %spill operations
    elseif Vessels(i) > 0 && max(Refuel(i,:)) > 0 && EmergencyEquipment(i) ~= 0
        %If vessel is in port, add 4 hours delay
        if OpStatus(i) == 1
            Delay = 4;
        %If vessel is in transit towards platform
        elseif OpStatus(i) == 2
            Delay = 0;
        %if vessel is sailing return trip, add 0.2 h. Should also
        %contribute to fuel consumption?
        elseif OpStatus(i) == 3
            Delay = 0.2;
        end
        ResponseDirect(i) = 0;
        ResponseRefuel(i) = SailingTimeOilSpill(i) + Delay;
        ResponseByPort(i) = 0;
    %If the current fuel level is so low that the ship are both unable to
    %perform oil spill operations independently, and unable to wait for
    %another ship to refuel, then it must travel back to port and refuel.
    %This option is also chosen if the vessel does not have the
    %necessary equipment onboard to deal with the emergency, and must thus
    %pick this up at port.
    else
        %If vessel is in port, add 4 hours delay
        if OpStatus(i) == 1
            Delay = TimeRefuelPort(i);
            ResponseByPort(i) = SailingTimePortOil(i);
        %If vessel is in transit towards platform, turn around
        elseif OpStatus(i) == 2
            Delay = 0.2;
            ResponseByPort(i) = SailingTimePort(i) + TimeRefuelPort(i) + SailingTimePortOil(i);
        %if vessel is sailing return trip
        elseif OpStatus(i) == 3
            Delay = 0;
            ResponseByPort(i) = SailingTimePort(i) + TimeRefuelPort(i) + SailingTimePortOil(i);
        end
        SailingTimeOilSpill(i) = ResponseByPort(i);
        
        ResponseDirect(i) = 0;
        ResponseRefuel(i) = 0;
        ResponseByPort(i) = SailingTimeOilSpill(i) + Delay;
    end
    ResponseTime(i) = SailingTimeOilSpill(i) + Delay;
end
      
%Choose the minimum response time of all vessels as the solution, 
%and print to command window the time and vessel chosen
     
[MinResponseTime, ChosenVessel] = min(ResponseTime(:))

%Initialize variable
RefueledByVessel = 0;
RefuelsAtPort = 0;

%Checking if the solution includes refueling by other vessel, and if so,
%which vessel to set to this task
for i=1:nVessels
    if MinResponseTime == ResponseRefuel(i)
        %Initialize the response time matrix of the fuel support vessels
        MinFuelSupportResponse = zeros(1,nVessels);
        %Checking the response time for the fuel support vessels
        for j=1:nVessels
            if Refuel(i,j) > 0
                MinFuelSupportResponse(j) = SailingTimePort(j) + TimeRefuelPort(j) + SailingTimePortOil(j);
            end
        end
        %The vessel responding to the oil spill can not be chosen to act
        %as fuel support for itself, and is thus set with a response time
        %ten times that of the maximum response time for all fuel
        %supporting vessels
        for j=1:nVessels
            if MinFuelSupportResponse(j) == 0
                MinFuelSupportResponse(j) = max(MinFuelSupportResponse(:))*10;
            end
        end
        %The vessel that can arrive first with extra fuel is chosen
        [MinFuelSupportResponseTime, FuelSupportVessel] = min(MinFuelSupportResponse(:));
        RefueledByVessel = FuelSupportVessel
        
    elseif MinResponseTime == ResponseDirect(i)
        disp('No refueling needed')
    elseif MinResponseTime == ResponseByPort(i)
        disp('Refuels at port')
        RefuelsAtPort = 1;
    end
end
      
%Plot the chosen vessel and its route to the oil spill
figuremap = figure('color','w');
ha = axesm('mapproj','mercator','maplatlim',[70 78],'maplonlim',[50 90]);
axis off, gridm off, framem on;
mlabel on, plabel on;

%Loading coastline data
load coast
hg = geoshow(lat, long,'displaytype','line','color','b');
%Color land areas green
geoshow('landareas.shp', 'FaceColor', [0.15 0.5 0.15])

%Mark the locations of interest
for i=1:nVessels
    if i == ChosenVessel
        geoshow(Latitude(i),Longitude(i),'Marker','o','Color','red');
        %Label the location of the chosen vessel
        textm(Latitude(ChosenVessel)-0.3,Longitude(ChosenVessel)-3, 'Chosen Vessel');
    elseif i == RefueledByVessel
        geoshow(Latitude(i),Longitude(i),'Marker','o','Color','green');
        %Label the location of the fuel support vessel
        textm(Latitude(i)-0.3,Longitude(i)-5, 'Fuel Support Vessel');
        %Plot the great circle route for the vessel chosen for fuel support
        GreatCirc = track2('gc',Latitude(FuelSupportVessel),Longitude(FuelSupportVessel),LocationPort(1),LocationPort(2));
        geoshow(GreatCirc(:,1),GreatCirc(:,2),'DisplayType','line','Color','green','linestyle','-');
        GreatCirc = track2('gc',LocationPort(1),LocationPort(2),LocationOilSpill(1),LocationOilSpill(2));
        geoshow(GreatCirc(:,1),GreatCirc(:,2),'DisplayType','line','Color','green','linestyle','-');
    else
        geoshow(Latitude(i),Longitude(i),'Marker','x','Color','black');
        %Label the location of the remaining vessels
        textm(Latitude(i)-0.3,Longitude(i)-3.5, 'Other Vessel');
    end
end

geoshow(LocationOilSpill(1),LocationOilSpill(2),'Marker','o','Color','black');
%Label the location of the oil spill
textm(LocationOilSpill(1)+0.3,LocationOilSpill(2)-2, 'Oil Spill');

geoshow(LocationOilSpill(1),LocationOilSpill(2),'Marker','.','Color','yellow');
%Label the location of the port
textm(LocationPort(1)-0.25,LocationPort(2)-0.2, 'Port');

%Create the great circle route from the chosen vessel to the oil spill
%If the vessel requires a stop at port
if RefuelsAtPort == 1
    GreatCirc = track2('gc',Latitude(ChosenVessel),Longitude(ChosenVessel),LocationPort(1),LocationPort(2));
    geoshow(GreatCirc(:,1),GreatCirc(:,2),'DisplayType','line','Color','red','linestyle','-');
    GreatCirc = track2('gc',LocationPort(1),LocationPort(2),LocationOilSpill(1),LocationOilSpill(2));
    geoshow(GreatCirc(:,1),GreatCirc(:,2),'DisplayType','line','Color','red','linestyle','-');
%If sailing directly to the oil spill
else
    GreatCirc = track2('gc',Latitude(ChosenVessel),Longitude(ChosenVessel),LocationOilSpill(1),LocationOilSpill(2));
    geoshow(GreatCirc(:,1),GreatCirc(:,2),'DisplayType','line','Color','red','linestyle','-');
end

%Closes excel, which have been held open through the entire simulation
Excel.ActiveWorkbook.Save; 
Excel.Quit 
Excel.delete 
clear Excel